#include <zlib.h>
#include <string>
using std::string;
#include <iostream>
using std::cout;
using std::endl;
using std::cerr;



bool writeGZ(gzFile file, void *ptr, size_t nbytes)
{
    if (gzwrite(file, ptr, nbytes) != (int)nbytes)
    {
        gzclose(file);
        return false;
    }
    return true;
}


bool loadGZ(gzFile file, void *ptr, size_t nbytes)
{
    if (gzread(file, ptr, nbytes) != (int)nbytes)
    {
        gzclose(file);
        return false;
    }
    return true;
}


template<typename T>
bool saveArray(string filename, T *a, size_t Nelements)
{
	if (a == NULL)
	{
	    return false;
	}
    gzFile file = gzopen(filename.c_str(), "wb9");
    if (!file)
    {
        return false;
    }

    // Write data to file
    if (!writeGZ(file, &Nelements, sizeof(size_t)))
    {
        return false;
    }
    if (!writeGZ(file, a, Nelements*sizeof(T)))
    {
        return false;
    }

    gzclose(file);
    return true;
}


template<typename T>
bool loadArray(string filename, T *a, size_t NelementsActual)
{
    if (!a)
    {
    	cout << "ERROR loadArray(): array must be allocated" << endl;
    	return false;
    }
    gzFile file = gzopen(filename.c_str(), "rb");
    if (file == NULL)
    {
        return false;
    }

    // Read data from file
    size_t NelementsInFile = 0;
    if (!loadGZ(file, &NelementsInFile, sizeof(size_t)))
    {
        return false; 
    }
    if (NelementsInFile == 0)
    {
    	cout << "ERROR loadArray(): No array elements in file" << endl;
        gzclose(file);
    	return false;
    }
    if (NelementsInFile != NelementsActual)
    {
    	cout << "ERROR loadArray(): Number of elements in file = " << NelementsInFile << ", but should be " << NelementsActual << endl;
        gzclose(file);
    	return false;
    }
    if (!loadGZ(file, a, NelementsInFile * sizeof(T)))
    {
        return false;
    }

    gzclose(file);
    return true;
}


int main()
{
    int n = 10;
    float *a = new float[n];
    for (int i = 0; i < n; i++) a[i] = (float)i;
    string filename = "test.gz";
    saveArray(filename, a, n);
    
    float *b = new float[n];
    loadArray(filename, b, n);
    bool ok = true;
    for (int i = 0; i < n; i++)
    {
        if (a[i] != b[i]) { ok = false; break; }
    }
    cout << "TEST " << (ok? "PASSED" : "FAILED") << endl;
    
    delete[] a;
    delete[] b;
}

